
import numpy as np
import matplotlib.pyplot as plt
import csv
import os
import statistics


def plot_distance_from_baseline():

    """
    This function reads x coordinates from a csv file to calculate the difference in positions of the detected objects
    between the start and end of the video. The csv file should include x coordinates where one
    row represent the detected objects in one frame. The number of rows are the number of frames in your video.

    The file you are reading should be in a folder with the name on the form "color-OD-pulsation-replicate"
    and the name of the csv file with the x coordinates should be x_coordinates.csv.

    The code makes box plots with the measured median distances from baseline for each light setting with the color you specified.
    The standard deviations, standard error and median positions are printed.

    Variables
    ---------
    color: color of your light source
    name: The name of your plot
    minutes: the minute you would like to compare with the baseline
    n = number of frames in one minute of your video
    x = length of a pixel in x direction
    num_light: number of light setting you would like to compare
    path: Path to folder with detection results. The csv file "x_coordinates.csv"
    sould be located in (for example): Lysforsøkjan2019/Light10Results/Results/blue-OD1-P1-R1
    """

    color = "white"
    name = "White - Median distance from baseline_test"
    minutes = 2
    n = 960
    x = 0.01633
    num_light = 12
    path = "/users/mariasolstad/desktop/Master_stats/Lysforsøkjan2019/"
    dirs = os.listdir(path)
    


    green_OD1_P1_end = []
    green_OD1_P2_end = []
    green_OD1_P3_end = []
    green_OD1_P0_end = []


    green_OD3_P1_end = []
    green_OD3_P2_end = []
    green_OD3_P3_end = []
    green_OD3_P0_end = []


    green_OD5_P1_end = []
    green_OD5_P2_end = []
    green_OD5_P3_end = []
    green_OD5_P0_end = []



    for dir in dirs:
        if not dir == ".DS_Store":
            new_path = path + "/" + dir + "/Results"
            experiments = os.listdir(new_path)

            for experiment in experiments:  # Loops through each experiment per day

                """Retrieving x corrdinates from beginning and end of the video"""

                a = experiment.split("-")

                """OD1"""
                if a[0] == color and a[1] == "OD1" and a[2] == "P1":

                    file_path = new_path + "/" + experiment + "/x_coordinates.csv"
                    if os.path.exists(file_path):
                        print(file_path)

                        with open(file_path, "r") as f:
                            data = list(np.array(rec) for rec in csv.reader(f, delimiter=",") if not len(rec) == 0)
                            row_count = len(data)
                            data_start = data[0:n]
                            data_end = data[row_count - minutes*n:row_count-(minutes-1)*n]
                            start = np.array([[float(j)*x for j in i] for i in data_start])
                            end = np.array([[float(j)*x for j in i] for i in data_end])
                            start = np.concatenate(start)
                            median = np.median(start)
                            for i in range(len(end)):
                                median_end = np.median(end[i][:])
                                green_OD1_P1_end.append(median - median_end)

                if a[0] == color and a[1] == "OD1" and a[2] == "P2":
                    file_path = new_path + "/" + experiment + "/x_coordinates.csv"
                    if os.path.exists(file_path):
                        print(file_path)

                        with open(file_path, "r") as f:
                            data = list(np.array(rec) for rec in csv.reader(f, delimiter=",") if not len(rec) == 0)
                            row_count = len(data)
                            print("Row count", row_count)
                            data_start = data[0:n]
                            data_end = data[row_count - minutes*n:row_count-(minutes-1)*n]
                            start = np.array([[float(j)*x for j in i] for i in data_start])
                            end = np.array([[float(j)*x for j in i] for i in data_end])
                            start = np.concatenate(start)
                            median = np.median(start)
                            for i in range(len(end)):
                                median_end = np.median(end[i][:])
                                green_OD1_P2_end.append(median - median_end)

                if a[0] == color and a[1] == "OD1" and a[2]=="P3":
                    file_path = new_path + "/" + experiment + "/x_coordinates.csv"
                    if os.path.exists(file_path):
                        print(file_path)

                        with open(file_path, "r") as f:
                            data = list(np.array(rec) for rec in csv.reader(f, delimiter=",") if not len(rec) == 0)
                            row_count = len(data)
                            print("Row count", row_count)
                            data_start = data[0:n]
                            data_end = data[row_count - minutes*n:row_count-(minutes-1)*n]
                            start = np.array([[float(j)*x for j in i] for i in data_start])
                            end = np.array([[float(j)*x for j in i] for i in data_end])
                            start = np.concatenate(start)
                            median = np.median(start)
                            for i in range(len(end)):
                                median_end = np.median(end[i][:])
                                green_OD1_P3_end.append(median - median_end)

                if a[0] == color and a[1] == "OD1" and a[2] == "P0":
                    file_path = new_path + "/" + experiment + "/x_coordinates.csv"
                    if os.path.exists(file_path):
                        print(file_path)

                        with open(file_path, "r") as f:
                            # reader = csv.reader(f)
                            data = list(np.array(rec) for rec in csv.reader(f, delimiter=",") if not len(rec) == 0)
                            row_count = len(data)
                            print("Row count", row_count)
                            data_start = data[0:n]
                            data_end = data[row_count - minutes*n:row_count-(minutes-1)*n]
                            start = np.array([[float(j)*x for j in i] for i in data_start])
                            end = np.array([[float(j)*x for j in i] for i in data_end])
                            start = np.concatenate(start)
                            median = np.median(start)
                            for i in range(len(end)):
                                median_end = np.median(end[i][:])
                                green_OD1_P0_end.append(median - median_end)

                """OD3"""

                if a[0] == color and a[1] == "OD3" and a[2] == "P1":

                    file_path = new_path + "/" + experiment + "/x_coordinates.csv"
                    if os.path.exists(file_path):
                        print(file_path)

                        with open(file_path, "r") as f:
                            data = list(np.array(rec) for rec in csv.reader(f, delimiter=",") if not len(rec) == 0)
                            row_count = len(data)
                            print("Row count", row_count)
                            data_start = data[0:n]
                            data_end = data[row_count - minutes*n:row_count-(minutes-1)*n]
                            start = np.array([[float(j)*x for j in i] for i in data_start])
                            end = np.array([[float(j)*x for j in i] for i in data_end])
                            start = np.concatenate(start)
                            median = np.median(start)
                            for i in range(len(end)):
                                median_end = np.median(end[i][:])
                                green_OD3_P1_end.append(median - median_end)

                if a[0] == color and a[1] == "OD3" and a[2] == "P2":
                    file_path = new_path + "/" + experiment + "/x_coordinates.csv"
                    if os.path.exists(file_path):
                        print(file_path)

                        with open(file_path, "r") as f:
                            data = list(np.array(rec) for rec in csv.reader(f, delimiter=",") if not len(rec) == 0)
                            row_count = len(data)
                            print("Row count", row_count)
                            data_start = data[0:n]
                            data_end = data[row_count - minutes*n:row_count-(minutes-1)*n]
                            start = np.array([[float(j)*x for j in i] for i in data_start])
                            end = np.array([[float(j)*x for j in i] for i in data_end])
                            start = np.concatenate(start)
                            median = np.median(start)
                            for i in range(len(end)):
                                median_end = np.median(end[i][:])
                                green_OD3_P2_end.append(median - median_end)

                if a[0] == color and a[1] == "OD3" and a[2]=="P3":
                    file_path = new_path + "/" + experiment + "/x_coordinates.csv"
                    if os.path.exists(file_path):
                        print(file_path)

                        with open(file_path, "r") as f:
                            data = list(np.array(rec) for rec in csv.reader(f, delimiter=",") if not len(rec) == 0)
                            row_count = len(data)
                            print("Row count", row_count)
                            data_start = data[0:n]
                            data_end = data[row_count - minutes*n:row_count-(minutes-1)*n]
                            start = np.array([[float(j)*x for j in i] for i in data_start])
                            end = np.array([[float(j)*x for j in i] for i in data_end])
                            start = np.concatenate(start)
                            median = np.median(start)
                            for i in range(len(end)):
                                median_end = np.median(end[i][:])
                                green_OD3_P3_end.append(median - median_end)

                if a[0] == color and a[1] == "OD3" and a[2]=="P0":
                    file_path = new_path + "/" + experiment + "/x_coordinates.csv"
                    if os.path.exists(file_path):
                        print(file_path)

                        with open(file_path, "r") as f:
                            data = list(np.array(rec) for rec in csv.reader(f, delimiter=",") if not len(rec) == 0)
                            row_count = len(data)
                            print("Row count", row_count)
                            data_start = data[0:n]
                            data_end = data[row_count - minutes*n:row_count-(minutes-1)*n]
                            start = np.array([[float(j)*x for j in i] for i in data_start])
                            end = np.array([[float(j)*x for j in i] for i in data_end])
                            start = np.concatenate(start)
                            median = np.median(start)
                            for i in range(len(end)):
                                median_end = np.median(end[i][:])
                                green_OD3_P0_end.append(median - median_end)
                """"OD5"""


                if a[0] == color and a[1] == "OD5" and a[2] == "P1":# and a[3] == replicate:

                    file_path = new_path + "/" + experiment + "/x_coordinates.csv"
                    if os.path.exists(file_path):
                        print(file_path)

                        with open(file_path, "r") as f:
                            data = list(np.array(rec) for rec in csv.reader(f, delimiter=",") if not len(rec) == 0)
                            row_count = len(data)
                            print("Row count", row_count)
                            data_start = data[0:n]
                            data_end = data[row_count - minutes*n:row_count-(minutes-1)*n]
                            start = np.array([[float(j)*x for j in i] for i in data_start])
                            end = np.array([[float(j)*x for j in i] for i in data_end])
                            start = np.concatenate(start)
                            median = np.median(start)
                            for i in range(len(end)):
                                median_end = np.median(end[i][:])
                                green_OD5_P1_end.append(median - median_end)

                if a[0] == color and a[1] == "OD5" and a[2] == "P2":
                    file_path = new_path + "/" + experiment + "/x_coordinates.csv"
                    if os.path.exists(file_path):
                        print(file_path)

                        with open(file_path, "r") as f:
                            data = list(np.array(rec) for rec in csv.reader(f, delimiter=",") if not len(rec) == 0)
                            row_count = len(data)
                            print("Row count", row_count)
                            data_start = data[0:n]
                            data_end = data[row_count - minutes*n:row_count-(minutes-1)*n]
                            start = np.array([[float(j)*x for j in i] for i in data_start])
                            end = np.array([[float(j)*x for j in i] for i in data_end])
                            start = np.concatenate(start)
                            median = np.median(start)
                            for i in range(len(end)):
                                median_end = np.median(end[i][:])
                                green_OD5_P2_end.append(median - median_end)

                if a[0] == color and a[1] == "OD5" and a[2]=="P3":
                    file_path = new_path + "/" + experiment + "/x_coordinates.csv"
                    if os.path.exists(file_path):
                        print(file_path)

                        with open(file_path, "r") as f:
                            data = list(np.array(rec) for rec in csv.reader(f, delimiter=",") if not len(rec) == 0)
                            row_count = len(data)
                            print("Row count", row_count)
                            data_start = data[0:n]
                            data_end = data[row_count - minutes*n:row_count-(minutes-1)*n]
                            start = np.array([[float(j)*x for j in i] for i in data_start])
                            end = np.array([[float(j)*x for j in i] for i in data_end])
                            start = np.concatenate(start)
                            median = np.median(start)
                            for i in range(len(end)):
                                median_end = np.median(end[i][:])
                                green_OD5_P3_end.append(median - median_end)

                if a[0] == color and a[1] == "OD5" and a[2]=="P0":
                    file_path = new_path + "/" + experiment + "/x_coordinates.csv"
                    if os.path.exists(file_path):
                        print(file_path)

                        with open(file_path, "r") as f:
                            data = list(np.array(rec) for rec in csv.reader(f, delimiter=",") if not len(rec) == 0)
                            row_count = len(data)
                            print("Row count", row_count)
                            data_start = data[0:n]
                            data_end = data[row_count - minutes*n:row_count-(minutes-1)*n]
                            start = np.array([[float(j)*x for j in i] for i in data_start])
                            end = np.array([[float(j)*x for j in i] for i in data_end])
                            start = np.concatenate(start)
                            median = np.median(start)
                            for i in range(len(end)):
                                median_end = np.median(end[i][:])
                                green_OD5_P0_end.append(median - median_end)

    plt.figure(figsize=(13, 6))
    plt.rcParams["figure.figsize"] = (13, 6)

    """Plotting constants"""

    placing = np.arange(num_light)
    placing_text = placing

    """Boxplot green OD"""
    data_to_plot_g = [green_OD1_P1_end, green_OD1_P2_end, green_OD1_P3_end, green_OD1_P0_end, green_OD3_P1_end, green_OD3_P2_end, green_OD3_P3_end, green_OD3_P0_end, green_OD5_P1_end, green_OD5_P2_end,  green_OD5_P3_end, green_OD5_P0_end]
    std_list = []
    median_list = []
    std_relative = []
    std_error = []
    for i in range(len(data_to_plot_g)):
        std = statistics.stdev(data_to_plot_g[i])
        median = np.median(data_to_plot_g[i])
        std_list.append(std)
        median_list.append(median)
        std_relative.append(std/median)
        std_error.append(std/(len(data_to_plot_g[i])))
    print("Standard_deviaitons", std_list)
    print("SE std/sqrt(n)", std_error)
    print("median list", median_list)


    plt.figure(7)

    plt.boxplot(data_to_plot_g, sym="o", whis = 1.5, positions=placing, widths=0.6)#,showmedians=True, medianline=True)
    plt.xticks(placing_text, ["OD1-P1", "OD1-P2", "OD1-P3", "OD1-P0", "OD3-P1", "OD3-P2", "OD3-P3", "OD3-P0", "OD5-P1", "OD5-P2", "OD5-P3", "OD5-P0"], fontsize=12)
    medians = [np.median(x) for x in data_to_plot_g]

    for i in range(len(medians)):
        plt.text(placing_text[i]-0.2, medians[i], str(np.round(medians[i], 2)), fontsize=13)
        plt.text(placing_text[i], medians[i] + 2, str(np.round(std_list[i],2)), fontsize=13)
    plt.title(name)
    plt.ylabel("Distance from baseline [cm]")
    plt.savefig(name + "_OD_P_medians_test.eps")


def main():

     plot_distance_from_baseline()


if __name__ == "__main__":
    # execute main
    main()
